package com.yizhi.wechat.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.wechat.application.vo.wechat.domain.PublicPlatformConfigVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;


/**
 * 配置信息feign接口
 *
 * @author lly
 * @date 2018-12-5
 */
@FeignClient(name = "wechat", contextId = "PublicConfigClient")
public interface PublicConfigClient {

    /**
     * 添加服务号的配置信息
     *
     * @param publicPlatformConfig
     * @return
     */
    @PostMapping(value = "/publicPlatformConfig/insert")
    PublicPlatformConfigVo insertPublicPlatformConfigInfo(@RequestBody PublicPlatformConfigVo publicPlatformConfig);

    /**
     * 查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/publicPlatformConfig/get")
    PublicPlatformConfigVo getPublicPlatformConfigInfo(@RequestParam(value = "id") Long id);

    /**
     * 修改
     *
     * @param publicPlatformConfig
     * @return
     */
    @PostMapping(value = "/publicPlatformConfig/update")
    PublicPlatformConfigVo updatePublicPlatformConfigInfo(@RequestBody PublicPlatformConfigVo publicPlatformConfig);

    /**
     * 查询列表
     *
     * @param pageSize
     * @param pageNo
     * @param name
     * @return
     */
    @GetMapping(value = "/publicPlatformConfig/list")
    Page<PublicPlatformConfigVo> getPublicPlatformConfigs(@RequestParam(value = "pageNo", required = true, defaultValue = "1") Integer pageNo,
                                                        @RequestParam(value = "pageSize", required = true, defaultValue = "10") Integer pageSize,
                                                        @RequestParam(value = "name") String name);

    /**
     * 删除配置信息
     *
     * @param ids
     * @return
     */
    @GetMapping(value = "/publicPlatformConfig/delete")
    boolean deletePublicPlatformConfig(@RequestBody List<Long> ids);

    /**
     * 获取配置信息
     *
     * @param publicPlatformConfig
     * @return
     */
    @PostMapping(value = "/publicPlatformConfig/getInfo")
    PublicPlatformConfigVo findPublicPlatformConfig(@RequestBody PublicPlatformConfigVo publicPlatformConfig);


}
