package com.yizhi.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.domain.UserInfo;
import com.yizhi.application.orm.util.DomainConverter;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.ITrAccountUserinfoService;
import com.yizhi.application.service.IUserInfoService;
import com.yizhi.application.service.impl.UserInfoServiceImpl;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.wechat.application.vo.wechat.AccountVO;
import com.yizhi.wechat.application.vo.wechat.AuthorizeUsersVO;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.wechat.application.vo.wechat.ResponseVO;
import com.yizhi.wechat.application.vo.wechat.TrAccountUserinfoVO;
import com.yizhi.wechat.application.vo.wechat.WechatUserInfoVO;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.ArrayUtils;
import org.apache.poi.util.ArrayUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lilingye
 * @since 2018-04-29
 */
@Log4j2
@RestController
@RequestMapping("/trAccountUserinfo")
public class TrAccountUserinfoController {

    @Autowired
    private ITrAccountUserinfoService iTrAccountUserinfoService;

    @Autowired
    private AccountClient accountClient;

    @Autowired
    private UserInfoController userInfoController;

    @Autowired
    private IUserInfoService userInfoService;

    @Autowired
    DomainConverter domainConverter;
    /**
     * 保存微信id和系统用户id的关系
     *
     * @param wechatUserId 用户的微信di
     * @param accountId    系统账号的id
     * @return TrAccountInfo
     */
    @RequestMapping(value = "/save/accountinfo", method = RequestMethod.GET)
    public TrAccountUserinfoVO saveTrAccountUserinfo(@RequestParam(value = "wechatUserId", required = true) String wechatUserId,
                                                     @RequestParam(value = "unionId", required = false) String unionId,
                                                     @RequestParam(value = "accountId", required = true) Long accountId,
                                                     @RequestParam(value = "siteId", required = true) Long siteId,
                                                     @RequestParam(value = "companyId", required = true) Long companyId,
                                                     @RequestParam(value = "terminalType", required = false) Integer terminalType
    ) {
        log.info("保存微信wechatId 和未木云账号的关系：");
        log.info("参数wechatuserid：" + wechatUserId + "；参数accountId:" + accountId + ";参数siteId：" + siteId);
        log.info("companyId：" + companyId);

        TrAccountUserinfoVO trAccountUserinfoVO = new TrAccountUserinfoVO();
        trAccountUserinfoVO.setWechatuserid(wechatUserId);
        trAccountUserinfoVO.setAccountId(accountId);
        trAccountUserinfoVO.setSiteId(siteId);
        trAccountUserinfoVO.setCompanyId(companyId);
        trAccountUserinfoVO.setType(terminalType);

        return iTrAccountUserinfoService.saveAccount(trAccountUserinfoVO);
    }

    /**
     * 解绑用户和微信的绑定关心
     *
     * @param ids 用户id
     * @return
     */
    @PostMapping(value = "/del/relations")
    public ResponseVO delRelation(@RequestBody List<Long> ids) {


        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        TrAccountUserinfo accountUserinfo = new TrAccountUserinfo();
        List<TrAccountUserinfo> trAccountUserinfoList = new ArrayList<TrAccountUserinfo>();
        // 初始化存放用户id的
        List<Long> accountIds = new ArrayList<Long>();
        Date date = new Date();
        for (Long id : ids
        ) {
            accountUserinfo.setAccountId(id);
            accountUserinfo.setIsDeleted(0);
            trAccountUserinfo = iTrAccountUserinfoService.selectOne(QueryUtil.condition(accountUserinfo));
            if (trAccountUserinfo != null) {
                trAccountUserinfo.setIsDeleted(1);
                trAccountUserinfo.setUpdateTime(date);
                trAccountUserinfoList.add(trAccountUserinfo);
            } else {
                accountIds.add(id);
            }
        }
        StringBuffer sb = new StringBuffer();
        sb.append("共操作").append(ids.size()).append("条,")
                .append("成功").append(ids.size() - accountIds.size()).append("条,")
                .append("失败").append(accountIds.size()).append("条。");


        //查询用户的列表
        List<AccountVO> accountVOS = null;
        if (accountIds.size() > 0) {
            List<com.yizhi.system.application.vo.AccountVO> accountVOS1 = accountClient.findByIds(accountIds);
            accountVOS = domainConverter.toDOList(accountVOS1,AccountVO.class);
        }


        if (CollectionUtils.isNotEmpty(accountVOS)) {
            sb.append("失败原因：");
            for (int i = 0; i < accountVOS.size(); i++) {
                if (i != accountVOS.size()) {
                    sb.append(accountVOS.get(i).getName()).append(",");
                } else {
                    sb.append(accountVOS.get(i).getName());
                }
            }
            sb.append(" 未绑定微信,不需要解绑!");
        }
        // 初始化返回对象
        ResponseVO responseVO = new ResponseVO();


        if (CollectionUtils.isNotEmpty(trAccountUserinfoList)) {
            boolean flag = iTrAccountUserinfoService.updateBatchById(trAccountUserinfoList);
            responseVO.setCode("1");
            responseVO.setMsg(sb.toString());
            return responseVO;
        } else {
            responseVO.setCode("0");
            responseVO.setMsg("您操作解绑的账号均未绑定微信，请重新选择!");
            return responseVO;
        }
    }

    /**
     * 通过用户的id获取微信openId
     *
     * @param accountId
     * @return
     */
    @GetMapping(value = "/get/wechatid")
    public String getWechatIdbyAccountId(@RequestParam(name = "accountId") Long accountId) {
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setIsDeleted(0);
        trAccountUserinfo.setAccountId(accountId);
        trAccountUserinfo = this.iTrAccountUserinfoService.selectOne(QueryUtil.condition(trAccountUserinfo));
        if (trAccountUserinfo != null) {
            return trAccountUserinfo.getWechatuserid();
        } else {
            return null;
        }


    }

    @PostMapping(value = "/relation/get")
    public TrAccountUserinfo getAccountUserInfo(@RequestBody TrAccountUserinfo trAccountUserinfo) {

        trAccountUserinfo.setIsDeleted(0);
        EntityWrapper<TrAccountUserinfo> condition = QueryUtil.condition(trAccountUserinfo);
        List<TrAccountUserinfo> trAccountUserinfos = iTrAccountUserinfoService.selectList(condition);
        if (CollectionUtils.isEmpty(trAccountUserinfos)){
            return null;
        }
        return trAccountUserinfos.get(0);
    }

    /**
     * 保存wechatId和accountId
     *
     * @param userInfoVO
     * @return
     */
    @PostMapping(value = "/save")
    public TrAccountUserinfoVO saveWechatUserInfo(@RequestBody TrAccountUserinfoVO userInfoVO) {
        return iTrAccountUserinfoService.saveAccount(userInfoVO);
    }

    /**
     * 修改关联关系
     *
     * @param trAccountUserinfo
     * @return
     */
    @PostMapping(value = "/stop")
    public Boolean updateTrAccountUserInfo(@RequestBody TrAccountUserinfo trAccountUserinfo) {

        trAccountUserinfo.setIsDeleted(0);
        EntityWrapper<TrAccountUserinfo> condition = QueryUtil.condition(trAccountUserinfo);
        TrAccountUserinfo userInfo = iTrAccountUserinfoService.selectOne(condition);

        if (userInfo != null) {
            return iTrAccountUserinfoService.updateTrAccountUserInfo(userInfo);
        } else {
            return true;
        }
    }

    /**
     * 根据微信weChatUserId解绑
     *
     * @param weChatUserId
     * @return
     */
    @PostMapping(value = "/unbind/weChatUserId")
    public Boolean unbindAccountByWeChatUserId(@RequestParam(name = "weChatUserId", required = true) String weChatUserId) {
        return iTrAccountUserinfoService.unbindAccountByWeChatUserId(weChatUserId);
    }

    /**
     * 根据微信weChatUserId解绑
     *
     * @return
     */
    @PostMapping(value = "/unbind/batch/weChatUserId")
    public Boolean unbindAccount(
            @RequestParam(value = "code") String code,
            @RequestParam(value = "appId") String appId,
            @RequestParam(value = "agentId", required = false) String agentId) {
        WechatUserInfoVO userInfo = userInfoController.getUserInfo(code, appId, agentId,null);
        log.info("获取到的用户信息：" + userInfo);
        if (null != userInfo) {
            return iTrAccountUserinfoService.unbindAccountByWeChatUserId(userInfo.getWUserInfoVO().getWechatuserid());
        }
        return false;
    }

    /**
     * 批量解绑用户
     * @param accountIds
     * @return
     */
    @PostMapping(value = "/unbindUser")
    public Boolean unbindUser(@RequestBody List<Long> accountIds) {
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setIsDeleted(Constants.NO_FLAG);
        EntityWrapper<TrAccountUserinfo> condition = QueryUtil.condition(trAccountUserinfo);
        condition.in("account_id", accountIds);
        List<TrAccountUserinfo> trAccountUserInfoList = iTrAccountUserinfoService.selectList(condition);
        Date date = new Date();
        if (CollectionUtils.isNotEmpty(trAccountUserInfoList)) {
            trAccountUserInfoList.parallelStream().map(accountUser -> {
                accountUser.setIsDeleted(Constants.YES_FLAG);
                accountUser.setUpdateTime(date);
                return accountUser;
            }).collect(Collectors.toList());
            iTrAccountUserinfoService.updateBatchById(trAccountUserInfoList);
            return Boolean.TRUE;
        }
        return Boolean.FALSE;

    }

    /**
     * 获取用户的绑定关系
     * @param authorizeUsersVO
     * @return
     */

    @PostMapping(value = "/get/bind/relation")
    public Map<Long,TrAccountUserinfo> getUserRelations(@RequestBody AuthorizeUsersVO authorizeUsersVO) {
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setIsDeleted(Constants.NO_FLAG);
        trAccountUserinfo.setCompanyId(authorizeUsersVO.getRoleId());
        EntityWrapper<TrAccountUserinfo> condition = QueryUtil.condition(trAccountUserinfo);
        condition.in("account_id", authorizeUsersVO.getUserIds());
//        condition.ne("type",3);
        List<TrAccountUserinfo> trAccountUserInfoList = iTrAccountUserinfoService.selectList(condition);
        List<UserInfo> userInfos = null;
        if (!CollectionUtils.isEmpty(trAccountUserInfoList)) {
            List<String> userInfoWechatUserIds =
                    trAccountUserInfoList.stream().map(TrAccountUserinfo::getWechatuserid).collect(Collectors.toList());
            log.info("wechatUserId:{}", ArrayUtils.toString(userInfoWechatUserIds.toArray()));
            EntityWrapper<UserInfo> entityWrapper = new EntityWrapper<>();
            entityWrapper.eq("is_deleted", Constants.DEL_FLAG);
            entityWrapper.in("wechatuserid", userInfoWechatUserIds);
            userInfos = userInfoService.selectList(entityWrapper);
            //获取用户微信昵称信息
            log.info("userInfoIds:{}", ArrayUtils.toString(userInfos.toArray()));
        }
        if (!CollectionUtils.isEmpty(userInfos)){
            Map<String,String> userInfoMap = userInfos.stream().collect(
                    Collectors.toMap(UserInfo::getWechatuserid, UserInfo::getNickname,(s,a)->s)
            );
            for (TrAccountUserinfo accountUserinfos: trAccountUserInfoList) {
                log.info("用户昵称："+userInfoMap.get(accountUserinfos.getWechatuserid()));
                if (userInfoMap.get(accountUserinfos.getWechatuserid()) == null) {
                    accountUserinfos.setWechatuserid("");
                } else {
                    accountUserinfos.setWechatuserid(userInfoMap.get(accountUserinfos.getWechatuserid()));
                }
            }
        }
        Map<Long, TrAccountUserinfo> entityMap= trAccountUserInfoList.stream().collect(
                Collectors.toMap(TrAccountUserinfo::getAccountId, TrAccountUserinfo->TrAccountUserinfo, (s, a) -> s));
        return entityMap;

    }
}

