package com.yizhi.application.controller;

import com.alibaba.fastjson.JSON;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.domain.UserInfo;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.IAccessTokenService;
import com.yizhi.application.service.IPublicPlatformConfigService;
import com.yizhi.application.service.ITrAccountUserinfoService;
import com.yizhi.application.service.IUserInfoService;
import com.yizhi.system.application.vo.domain.Account;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.LoginLogClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.application.task.UpdateUserInfoService;
import com.yizhi.util.application.constant.ReturnCode;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.system.application.vo.LoginLogAddReqVO;
import com.yizhi.system.application.vo.WeChatUserVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 获取微信token
 *
 * @author lingye
 * @date 2020-2-10 17:02:14
 */
@RestController
@RequestMapping(value = "/wechat")
public class WeChatUserController {

    public static final Logger LOGGER = LoggerFactory.getLogger(WeChatUserController.class);

    @Autowired
    private SiteClient siteClient;
    @Autowired
    private AccountClient accountClient;
    @Autowired
    private ITrAccountUserinfoService iTrAccountUserinfoService;
    @Autowired
    private UserInfoController userInfoController;
    @Autowired
    private IPublicPlatformConfigService iPublicPlatformConfigService;
    @Autowired
    private UpdateUserInfoService updateUserInfoService;
    @Autowired
    private LoginLogClient loginLogClient;
    @Autowired
    private IAccessTokenService accessTokenService;
    @Autowired
    private IUserInfoService userInfoService;


    @GetMapping("/appid/accesstoken/get")
    public String getWechatAccessTokenByAppId(@RequestParam("appId") String appId,
                                              @RequestParam(value = "agentId",required = false) String agentId) {
        PublicPlatformConfig config =iPublicPlatformConfigService.getPubConfig(appId,agentId);
        return accessTokenService.getAccessToken(config);
    }

    @GetMapping(value = "/token")
    public Map<String, Object> getWeChatUserInfoToken(@RequestParam(name = "weChatUserId", required = true) String weChatUserId,
                                                      @RequestParam(name = "appId", required = true) String appId,
                                                      @RequestParam(name = "agentId", required = false) String agentId
    ) {
        RequestContext context = ContextHolder.get();
        LOGGER.info("请求的上下文信息:{}", JSON.toJSON(context));
        Map<String, Object> retMap = new HashMap<String, Object>(16);
        retMap.put("returnCode", 1);
        //获取企业的信息
        Long siteId = context.getSiteId();
        String siteCode = context.getSiteCode();
        Long companyId = context.getCompanyId();
        String companyCode = context.getCompanyCode();


        PublicPlatformConfig platformConfig = null;
        List<PublicPlatformConfig> pubConfigs = iPublicPlatformConfigService.getPubConfigs(appId, agentId);
        LOGGER.info("公众号配置所有列表：{}", JSON.toJSON(pubConfigs));
        if (pubConfigs == null) {
            retMap.put("returnCode", 2);
            return retMap;
        }
        for (PublicPlatformConfig config : pubConfigs) {
            if (companyCode.equals(config.getCompanyCode()) && siteCode.equals(config.getSiteCode())) {
                platformConfig = config;
                break;
            }
        }
        if (platformConfig == null) {
            // 公众号配置的站点和链接的站点不匹配
            retMap.put("returnCode", 2);
            retMap.put("msg","配置链接的公司站点编码和微信公众号配置的不一致！");
            return retMap;
        }
        LOGGER.info("公众号配置：{}", JSON.toJSON(platformConfig));
        TrAccountUserinfo trAccountUserinfo = userInfoController.queryUserInWechat(weChatUserId, companyId);
        // 如果是光大证券的特殊要命需求：
        // 1.   没有找到 就去system用户表里面找
        // 1.1  先查询当前企业用户的信息
        if (companyCode.equals("ebscn")) {
            if (StringUtils.isNotEmpty(platformConfig.getAgentId())) {
                String accessToken = userInfoController.getWeChatAccessToken(platformConfig);
                // 查询用户的信息
                com.yizhi.wechat.application.vo.wechat.WeChatUserVO weChatUserVO = userInfoController.getWechatUser(accessToken, weChatUserId);
                LOGGER.info("-----------------------------------------------------");
                LOGGER.info("-----------------------------------------------------");
                LOGGER.info("  测试光大用，后期删除： weChatUserVO：{}", JSON.toJSONString(weChatUserVO));
                LOGGER.info("-----------------------------------------------------");
                LOGGER.info("-----------------------------------------------------");
                if (weChatUserVO == null) {
                    retMap.put("returnCode", 2);
                    retMap.put("msg","请求企业微信中用户信息失败");
                    return retMap;
                }
                ResponseEntity<Account> accountResponseEntity =
                        accountClient.searchByConditionWithPriority(companyId, weChatUserId,
                        weChatUserVO.getAccountVO().getMobile(), weChatUserVO.getAccountVO().getEmail());
                Account account = accountResponseEntity.getBody();
                // 没有找到 提示报错
                if (account == null) {
                    retMap.put("returnCode", 10);
                    return retMap;
                } else {
                    bindCompanyWeChatUser(weChatUserId, companyId, siteId, account.getId(), 1);
                    AccountVO accountVO = new AccountVO();
                    BeanUtils.copyProperties(account, accountVO);
                    retMap.put("userInfo", accountVO);
                    return retMap;
                }
            }
            retMap.put("returnCode", 2);
            retMap.put("msg","微信配置agentId为空");
            return retMap;
        }

        if (null != trAccountUserinfo) {
            // 异步更新用户信息、部门信息
            updateUserInfoService.taskUpdateUserInfo(platformConfig, trAccountUserinfo);
            AccountVO accountVO = accountClient.findById(trAccountUserinfo.getAccountId());

            Map<String, Object> stringObjectMap = checkUser(accountVO);
            if (!stringObjectMap.isEmpty()){
                return stringObjectMap;
            }

            LOGGER.info("当前用户的信息accountVO：{}", accountVO);
            if (!getSiteAuthorize(accountVO.getId(), siteId, companyId, companyCode)) {
                // 没有站点访问权限
                LOGGER.info("没有访问权限！");
                retMap.put("returnCode", 0);
                return retMap;
            }
            retMap.put("userInfo", accountVO);
            addUserLoginLog(companyId, siteId, accountVO, Constants.THREE);
            return retMap;
        } else {
            LOGGER.info("不存在账号");
            LOGGER.info("公众号配置：{}", platformConfig);
            LOGGER.info("判断：{}", platformConfig.getIsAddedUser() == 1 ? true : false);
            AccountVO accountVO = null;
            if (Constants.ZERO.equals(platformConfig.getType()) && platformConfig.getIsAddedUser() == 1) {
                LOGGER.info("当前的结果===");
                accountVO = getCompanyWeChatUser(siteId, companyId, siteCode, weChatUserId, platformConfig, context);
                if (accountVO != null) {
                    retMap.put("userInfo", accountVO);
                    addUserLoginLog(companyId, siteId, accountVO, Constants.THREE);
                }
            }
            if (Constants.TWO.equals(platformConfig.getType())) {
                // 服务商接口
                accountVO = getProviderUser(siteId,companyId,weChatUserId,context);
                retMap.put("userInfo", accountVO);
                addUserLoginLog(companyId, siteId, accountVO, Constants.FIVE);
            }
            return retMap;
        }
    }
    private static Map<String, Object> checkUser(AccountVO accountVO) {

        Map<String, Object> retMap = new HashMap<String, Object>(16);

        if (!accountVO.getEnabled() || accountVO.getLocked()) {
            retMap.put("returnCode", 2);
            retMap.put("msg", ReturnCode.ACCOUNT_CLOSE.getMsg());
            // 前面时间大于后面时间==1
        } else if (accountVO.getExpiredType() == 2 && accountVO.getEndTime().compareTo(new Date()) <= 0) {
            retMap.put("returnCode", 2);
            retMap.put("msg", ReturnCode.ACCOUNT_INVALID.getMsg());
        } else if (accountVO.getExpiredType() == 3 && accountVO.getExpiredTime().compareTo(new Date()) <= 0) {
            retMap.put("returnCode", 2);
            retMap.put("msg", ReturnCode.ACCOUNT_INVALID.getMsg());
        }
        return retMap;
    }
    /**
     * 企业微信新增
     *
     * @param siteId
     * @param companyId
     * @param siteCode
     * @param weChatUserId
     * @param publicPlatformConfig
     * @return
     */
    public AccountVO getCompanyWeChatUser(
            Long siteId, Long companyId, String siteCode, String weChatUserId, PublicPlatformConfig publicPlatformConfig
            , RequestContext context) {
        LOGGER.info("添加用户上下参数：{}", JSON.toJSON(context));
        String companyCode = context.getCompanyCode();
        try {
            AccountVO userInfo = accountClient.findUser(weChatUserId, companyId);
            // 判断用户是否存在
            if (null != userInfo) {
                LOGGER.info("存在同名的用户。");
                bindCompanyWeChatUser(weChatUserId, companyId, siteId, userInfo.getId(),0);
                return userInfo;
            } else {
                String accessToken = accessTokenService.getAccessToken(publicPlatformConfig);
                LOGGER.info("获取微信用户信息token：{}", accessToken);
                com.yizhi.wechat.application.vo.wechat.WeChatUserVO  weChatUser = userInfoController.getWechatUser(accessToken, weChatUserId);

                if (weChatUser == null) {
                    return null;
                }
                weChatUser.setContext(context);
                WeChatUserVO weChatUserVO = new WeChatUserVO();
                BeanUtils.copyProperties(weChatUser,weChatUserVO);
                // 深度复制
                com.yizhi.wechat.application.vo.wechat.AccountVO accountVOTemp = weChatUser.getAccountVO();
                com.yizhi.system.application.vo.AccountVO accountVOTemp1 = new AccountVO();
                BeanUtils.copyProperties(accountVOTemp,accountVOTemp1 );
                weChatUserVO.setAccountVO(accountVOTemp1);
                LOGGER.info("获取的用户微信信息{}", JSON.toJSON(weChatUser));

                LOGGER.info("获取的用户微信信息，负责的信息 = {}", JSON.toJSON(weChatUserVO));
                Account account = null;
                try {
                    account = accountClient.saveUser(weChatUserVO);
                    LOGGER.info("添加后的用户信息：{}", JSON.toJSON(account));
                } catch (Exception e) {
                    LOGGER.info("保存异常：{}", JSON.toJSONString(e));
                }
                // 绑定微信号
                bindCompanyWeChatUser(weChatUserId, companyId, siteId, account.getId(),0);
                AccountVO accountVO = new AccountVO();
                BeanUtils.copyProperties(account, accountVO);
                // 生成站点访问权限
                Boolean resultFlag = siteClient.relateAccountSite(siteId, account.getId(), companyCode);

                LOGGER.info("访问权限生成:" + resultFlag);
                return accountVO;
            }
        } catch (Exception e) {
            LOGGER.info("返回异常：{}", JSON.toJSONString(e));
            return null;
        }
    }


    public AccountVO getProviderUser(
            Long siteId, Long companyId, String weChatUserId, RequestContext context) {
        LOGGER.info("添加用户上下参数：{}", JSON.toJSON(context));
        String companyCode = context.getCompanyCode();
        try {
            WeChatUserVO weChatUser = new WeChatUserVO();
            UserInfo userInfo = new UserInfo();
            userInfo.setWechatuserid(weChatUserId);
            userInfo.setIsDeleted(0);
            userInfo = userInfoService.selectOne(QueryUtil.condition(userInfo));


            AccountVO accountVO = new AccountVO();
            accountVO.setName(userInfo.getNickname());
            accountVO.setHeadPortrait(userInfo.getHeadimgurl());
            accountVO.setName(weChatUserId);
            accountVO.setEmail(userInfo.getEmail());
            accountVO.setMobile(userInfo.getMobile());
            accountVO.setFullName(userInfo.getNickname());
            weChatUser.setAccountVO(accountVO);
            weChatUser.setContext(context);
            weChatUser.setCorpId(userInfo.getCorpId());

            Account account = null;
            try {
                account = accountClient.saveProviderUser(weChatUser);
                LOGGER.info("添加后的用户信息：{}", JSON.toJSON(account));
            } catch (Exception e) {
                LOGGER.info("保存异常：{}", e.getMessage());
            }
            if (account ==null){
                LOGGER.info("新增用户失败");
            }
            // 绑定微信号
            bindCompanyWeChatUser(weChatUserId, companyId, siteId, account.getId(),5);
               BeanUtils.copyProperties(account,accountVO);
            // 生成站点访问权限
            Boolean resultFlag = siteClient.relateAccountSite(siteId, account.getId(), companyCode);

            LOGGER.info("访问权限生成:" + resultFlag);
            return accountVO;
        } catch (Exception e) {
            LOGGER.info("返回异常：{}", e.getMessage());
            return null;
        }
    }

    /**
     * 绑定用户
     *
     * @param weChatUserId
     * @param companyId
     * @param siteId
     * @param accountId
     */
    @Transactional(rollbackFor = Exception.class)
    public void bindCompanyWeChatUser(String weChatUserId, Long companyId, Long siteId, Long accountId,Integer type) {
        TrAccountUserinfo trAccountUserInfo = new TrAccountUserinfo();
        trAccountUserInfo.setWechatuserid(weChatUserId);
        trAccountUserInfo.setAccountId(accountId);
        trAccountUserInfo.setSiteId(siteId);
        trAccountUserInfo.setCompanyId(companyId);
        trAccountUserInfo.setType(type);
        trAccountUserInfo.setCreateTime(new Date());
        LOGGER.info("企业用户绑定的参数：{}", JSON.toJSON(trAccountUserInfo));
        boolean bindUser = iTrAccountUserinfoService.insert(trAccountUserInfo);
        LOGGER.info("绑定企业号：{}", bindUser);
    }

    /**
     * 判断站点是否有权
     *
     * @param siteId
     * @return
     */
    public Boolean getSiteAuthorize(Long accountId, Long siteId, Long companyId, String companyCode) {
        LOGGER.info("判断权限接口：accountId:" + accountId + ",siteId：" + siteId + ",companyId:" + companyId);
        try {
            boolean accessToSite = siteClient.accessToSite(accountId, siteId, companyId, companyCode);
            LOGGER.info("返回的站点权限1：{}", accessToSite);
            if (accessToSite) {
                return Boolean.TRUE;
            }
            accessToSite = siteClient.accessToSite(accountId, siteId, companyId, null);
            LOGGER.info("返回的站点权限2：{}", accessToSite);
            return accessToSite;
        } catch (Exception e) {
            LOGGER.info("调用system异常：{}", e.getMessage());
            return false;
        }
    }

    /**
     * 用户登录统计
     *
     * @param companyId 公司id
     * @param siteId    站点id
     */
    public void addUserLoginLog(Long companyId, Long siteId, AccountVO accountVO, Integer type) {
        LoginLogAddReqVO loginLogAddReqVO = new LoginLogAddReqVO();
        loginLogAddReqVO.setCompanyId(companyId);
        loginLogAddReqVO.setSiteId(siteId);
        loginLogAddReqVO.setType(type);
        loginLogAddReqVO.setAccountId(accountVO.getId());
        loginLogAddReqVO.setOrgId(accountVO.getOrgId());

        try {
            loginLogClient.addLoginLog(loginLogAddReqVO);
        } catch (Exception e) {
            LOGGER.error("统计用户异常{}", e.getMessage());
        }
    }

}
