package com.yizhi.application.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.aes.AesException;
import com.yizhi.application.aes.WXBizMsgCrypt;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.SuiteTicket;
import com.yizhi.application.service.IPublicPlatformConfigService;
import com.yizhi.application.service.ISuiteTicketService;
import com.yizhi.wechat.application.utils.ElementUtils;
import com.yizhi.wechat.application.utils.FormatUtils;
import com.yizhi.wechat.application.utils.OkHttpUtils;
import com.yizhi.wechat.application.vo.wechat.SuiteTicketVO;
import com.yizhi.wechat.application.vo.wechat.WechatProviderVO;
import io.swagger.annotations.Api;
import lombok.extern.log4j.Log4j2;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Iterator;


/**
 * 服务商接口
 *
 * @author lingye
 */
@Api(tags = "微信供应商接口")
@RequestMapping("/provider")
@RestController
@Log4j2
public class WechatProviderController {

//    public final static String TOKEN = "a30rw9jBPYAmdjoxqqWgWfrYqdHP";
//    public final static String ENCODING_AESKEY = "TOAX0ShveDLbPGl4rZRdmFxemn7GXxxe0OO950bXvqz";
//    public final static String CORP_ID = "ww3cf43133c5880598";
//    public final static String SUITE_ID = "ww67f499ed2d11a2dc";

//    public final static String TOKEN1 = "kFziYBcchlLFknse";
//    public final static String ENCODING_AESKEY1 = "3uhcLd0Pmr53ObF3dAd4Bp9xhQDsWMieTcMKxdqKor3";

    @Value("${provider.corp.encoding.aeskey}")
    private String encodingAesKey;
    @Value("${provider.corp.token}")
    private String corpToken;
    @Value("${provider.corpId}")
    private String corpId;

    @Value("${provider.suite.id}")
    private String suiteId;
    @Value("${provider.suite.token}")
    private String suiteToken;
    @Value("${provider.suite.encoding.aseKey}")
    private String suiteEncodingAesKey;

//    HttpClientUtils httpClientUtils = new HttpClientUtils();

    @Autowired
    RedisCache redisCache;

    @Value("${wechat.provider.token}")
    private String providerAccessTokenUrl;


    @Value("${wechat.provider.suite.token}")
    private String suiteTokenUrl;


    @Autowired
    private IPublicPlatformConfigService publicPlatformConfigService;
    @Autowired
    private ISuiteTicketService suiteTicketService;

    /**
     * 通用开发参数 -系统事件接收URL
     *
     * @param signature 签名
     * @param timestamp 时间戳
     * @param nonce     随机数
     * @param echostr
     * @return
     * @throws AesException
     */
    @RequestMapping(value = "/get", method = RequestMethod.GET)
    public String checkToken(@RequestParam(name = "msg_signature") String signature,
                             @RequestParam(name = "timestamp") String timestamp,
                             @RequestParam(name = "nonce") String nonce,
                             @RequestParam(name = "echostr") String echostr
    ) throws AesException {
        //token和encoding_aeskey就是上诉随机获取的值，corp_id是企业id，在企业微信管理页面点击： 我的企业，拉到最下方可以看到
        WXBizMsgCrypt wxcpt = new WXBizMsgCrypt(corpToken, encodingAesKey, corpId);
        String sEchoStr = wxcpt.VerifyURL(signature, timestamp, nonce, echostr);
        System.out.println("-----签名校验通过-----");
        return sEchoStr;
    }

    /**
     * 服务商应用的
     *
     * @param signature
     * @param timestamp
     * @param nonce
     * @param echostr
     * @return
     * @throws AesException
     */
    @RequestMapping(value = "/callback", method = RequestMethod.GET)
    public String callBack(@RequestParam(name = "msg_signature") String signature,
                           @RequestParam(name = "timestamp") String timestamp,
                           @RequestParam(name = "nonce") String nonce,
                           @RequestParam(name = "echostr") String echostr
    ) throws AesException {
        //token和encoding_aeskey就是上诉随机获取的值，corp_id是企业id，在企业微信管理页面点击： 我的企业，拉到最下方可以看到
        WXBizMsgCrypt wxcpt = new WXBizMsgCrypt(suiteToken, suiteEncodingAesKey, corpId);
        String sEchoStr = wxcpt.VerifyURL(signature, timestamp, nonce, echostr);
        System.out.println("-----签名校验通过-----");
        return sEchoStr;
    }

    /**
     * 应用接受消息的接口
     *
     * @param wechatProviderVO
     * @return
     * @throws Exception
     */
    @PostMapping(value = "/handle", produces = {"application/xml;charset=UTF-8"})
    public Object handlerMessage(@RequestBody WechatProviderVO wechatProviderVO) throws Exception {

        WXBizMsgCrypt wxcpt = new WXBizMsgCrypt(suiteToken, suiteEncodingAesKey, corpId);
        String sReqData = FormatUtils.getReqData(wechatProviderVO.getMsg());

        String signature = wechatProviderVO.getSignature();
        String timestamp = wechatProviderVO.getTimestamp();
        String nonce = wechatProviderVO.getNonce();

        String sMsg = wxcpt.DecryptMsg(signature, timestamp, nonce, sReqData);
        ElementUtils element = new ElementUtils(sMsg);
        String msgType = element.get("MsgType");
        String content = "";
        switch (msgType) {
            /**
             * 事件类型处理
             */
            case "event":
                content = "触发事件";
                break;
            /**
             * 文字消息处理：这里是将用户发送的消息原样返回
             */
            case "text":
                content = element.get("Content");
                break;
            default:
                break;
        }
        String sRespData = FormatUtils.getTextRespData(content);
        String sEncryptMsg = wxcpt.EncryptMsg(sRespData, timestamp, nonce);
        return sEncryptMsg;
    }


    @GetMapping("/suitetoken")
    public String getSuiteToken(@RequestParam(name = "suiteId") String suiteId) {
        log.info("当前suite_id：{}",suiteId);
        Object hget = redisCache.hget("st_" + suiteId, "st_" + suiteId);
        log.info("返回的结果：{}",JSON.toJSON(hget));
        String suiteAccessToken = null;
        if (hget != null) {
            suiteAccessToken = hget.toString();
            log.info("token：{}", JSON.toJSON(hget));
            return suiteAccessToken;
        }
        PublicPlatformConfig pubConfig = publicPlatformConfigService.getPubConfig(suiteId, null);
        log.info("suite_token:{}",JSON.toJSON(pubConfig));
        SuiteTicket suiteTicket = suiteTicketService.getSuiteTicketById(suiteId);

        log.info("返回suiteTicket：{}", JSON.toJSON(suiteTicket));
        HashMap<String, String> map = new HashMap<>();
        map.put("suite_id", suiteId);
        map.put("suite_secret", pubConfig.getSecret());
        map.put("suite_ticket", suiteTicket.getSuiteTicket());
        log.info("参数map:{}",JSON.toJSON(map));
        HashMap<String, String> head = new HashMap<>();
        head.put("Content-Type", "application/json");
        String result = OkHttpUtils.doRequest2("https://qyapi.weixin.qq.com/cgi-bin/service/get_suite_token", head, JSON.toJSONString(map));
        log.info("请求后的token:{}",result);

        JSONObject suiteJson = JSON.parseObject(result);
        suiteAccessToken = suiteJson.getString("suite_access_token");
        log.info("suiteAccessToken：{}",suiteAccessToken);
        redisCache.hset("st_" + suiteId, "st_" + suiteId, suiteAccessToken, 7200);
        return suiteAccessToken;
    }

    /**
     * suite_ticket 的接收
     *
     * @param suiteTicketVO
     * @return
     * @throws AesException
     */
    @PostMapping(value = "/suite/receive", produces = {"application/xml;charset=UTF-8"})
    public String suiteReceive(@RequestBody SuiteTicketVO suiteTicketVO) throws AesException {

        log.info("参数:{}", JSON.toJSON(suiteTicketVO));

        WXBizMsgCrypt wxcpt = new WXBizMsgCrypt(suiteToken, suiteEncodingAesKey, suiteId);
        String sReqData = FormatUtils.getReqData(suiteTicketVO.getInMsgEntity());

        String msg_signature = suiteTicketVO.getMsg_signature();
        String timestamp = suiteTicketVO.getTimestamp();
        String nonce = suiteTicketVO.getNonce();

        String sMsg = wxcpt.DecryptMsg(msg_signature, timestamp, nonce, sReqData);

        log.info("返回的结果：{}", sMsg);


        SuiteTicket suiteTicket = new SuiteTicket();
        SuiteTicket s = updateSuiteTicket(sMsg, suiteTicket);
        if (s == null) {
            return "fail";
        }
        suiteTicketService.saveOrUpdateSuiteTicket(suiteTicket);

        return "success";
    }

    /**
     * @param xmlResult
     * @param suiteTicket
     * @return
     */
    private SuiteTicket updateSuiteTicket(String xmlResult, SuiteTicket suiteTicket) {
        try {
            Document document = DocumentHelper.parseText(xmlResult);
            Element rootElement = document.getRootElement();
            // 迭代xml字段节点
            Iterator iter = rootElement.elementIterator();
            // 获取节点

            while (iter.hasNext()) {
                Element elementOption = (Element) iter.next();
                // 节点属性
                String name = elementOption.getName();
                // 节点元素
                String text = elementOption.getText();
                log.info("当前" + name + "的值：{}", text);

                if (name.equals("SuiteId")) {
                    suiteTicket.setSuiteId(text);
                } else if (name.equals("SuiteTicket")) {
                    suiteTicket.setSuiteTicket(text);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return suiteTicket;

    }
}

