package com.yizhi.application.feign;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.wechat.application.vo.wechat.WechatParamVO;
import feign.Headers;
import feign.Param;
import feign.RequestLine;
import org.springframework.stereotype.Service;

/**
 * 获取企业微信用户信息
 *
 * @author lilingye
 * @since 2018-5-3 19:17:41
 */

@Headers("Content-Type: application/json")
@Service("cWeChatClient")
public interface CWeChatClient {
    /**
     * 通过code获取token
     *
     * @param secret
     * @return
     */
    @RequestLine("GET /cgi-bin/gettoken?corpsecret={secret}&corpid={corpid}")
    JSONObject getWechatToken(@Param("secret") String secret, @Param("corpid") String corpid);


    /**
     * #根据code和AccessToken获取用户信息
     *
     * @param access_token
     * @param code
     * @return
     */
    @RequestLine("GET /cgi-bin/user/getuserinfo?access_token={access_token}&code={code}")
    JSONObject getUserInfo(@Param("access_token") String access_token, @Param("code") String code);

    /**
     * 获取用户的详细信息
     *
     * @param wechatParamVO user_ticket信息
     * @return
     */
    @RequestLine("POST /cgi-bin/user/getuserdetail?access_token={access_token}")
    JSONObject getUserDetail(@Param("access_token") String access_token, WechatParamVO wechatParamVO);

    /**
     * 获取ticket
     *
     * @param access_token
     * @return
     */
    @RequestLine("GET /cgi-bin/get_jsapi_ticket?access_token={access_token}")
    JSONObject getTicket(@Param("access_token") String access_token);

    /**
     * 获取企业用户信息包括当前部门的id
     *
     * @param accessToken ACCESSTOKEN
     * @param userId      用户userId
     * @return 用户详细信息
     */
    @RequestLine("GET /cgi-bin/user/get?access_token={accessToken}&userid={userId}")
    JSONObject getCWechatUserDetail(@Param("accessToken") String accessToken, @Param("userId") String userId);

    /**
     * 获取企业服务号部门列表
     *
     * @param accessToken
     * @param id          部门id
     * @return 列表
     */
    @RequestLine("GET /cgi-bin/department/list?access_token={accessToken}&id={id}")
    JSONObject getCwechatDepartmentList(@Param("accessToken") String accessToken, @Param("id") Long id);


}
