package com.yizhi.application.feign;

import com.netflix.hystrix.HystrixCommandProperties;
import feign.hystrix.HystrixFeign;
import feign.hystrix.SetterFactory;
import feign.jackson.JacksonDecoder;
import feign.jackson.JacksonEncoder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Slf4j
@Configuration
public class WeChatConfig {


    @Value("${cloud.wechat.base_url}")
    private String baseUrl;

    private String accessTimeOutMillis = "1000";


    @Bean
    WeChatClient weChatClient() throws InterruptedException {
        try {
            Integer.parseInt(accessTimeOutMillis);
            return HystrixFeign.builder()
                    .decoder(new JacksonDecoder())
                    .encoder(new JacksonEncoder())
                    .setterFactory((target, method) ->
                            new SetterFactory.Default().create(target, method).
                                    andCommandPropertiesDefaults(HystrixCommandProperties.defaultSetter().
                                            withExecutionTimeoutInMilliseconds(Integer.parseInt(accessTimeOutMillis))))
                    .target(WeChatClient.class, this.baseUrl);
        } catch (Exception e) {
            return HystrixFeign.builder()
                    .decoder(new JacksonDecoder())
                    .encoder(new JacksonEncoder())
                    .setterFactory((target, method) ->
                            new SetterFactory.Default().create(target, method).
                                    andCommandPropertiesDefaults(HystrixCommandProperties.defaultSetter().
                                            withExecutionTimeoutInMilliseconds(10000)))
                    .target(WeChatClient.class, this.baseUrl);
        }
    }
}
