package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.mapper.PublicPlatformConfigMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.IPublicPlatformConfigService;
import com.yizhi.wechat.application.vo.wechat.Constants;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lilingye
 * @since 2018-04-29
 */
@Service
@Transactional
@Log4j2
public class PublicPlatformConfigServiceImpl extends ServiceImpl<PublicPlatformConfigMapper, PublicPlatformConfig> implements IPublicPlatformConfigService {

    @Autowired
    IdGenerator idGenerator;

    /**
     * 获取公众号配置信息
     *
     * @param appId 公众号的APPID
     * @return
     */
    @Override
    public PublicPlatformConfig getPubConfig(String appId, String agentId) {

        PublicPlatformConfig publicPlatformConfig = new PublicPlatformConfig();
        publicPlatformConfig.setAppId(appId);
        publicPlatformConfig.setAgentId(StringUtils.isNotEmpty(agentId) ? agentId.trim() : null);
        publicPlatformConfig.setIsDeleted(Constants.DEL_FLAG);
        return this.selectOne(QueryUtil.condition(publicPlatformConfig));
    }

    @Override
    public PublicPlatformConfig getConfigByAppId(String appId) {
        PublicPlatformConfig publicPlatformConfig = new PublicPlatformConfig();
        publicPlatformConfig.setAppId(appId);
        publicPlatformConfig.setIsDeleted(Constants.DEL_FLAG);
        List<PublicPlatformConfig> configList =  this.selectList(QueryUtil.condition(publicPlatformConfig));
        if (configList == null || configList.isEmpty()) {
            return null;
        }
        return configList.get(0);
    }

    @Override
    public List<PublicPlatformConfig> getPubConfigs(String appId, String agentId) {
        PublicPlatformConfig publicPlatformConfig = new PublicPlatformConfig();
        publicPlatformConfig.setAppId(appId);
        publicPlatformConfig.setAgentId(StringUtils.isNotEmpty(agentId) ? agentId.trim() : null);
        publicPlatformConfig.setIsDeleted(Constants.DEL_FLAG);
        List<PublicPlatformConfig> publicPlatformConfigs = this.selectList(QueryUtil.condition(publicPlatformConfig));
        if (CollectionUtils.isNotEmpty(publicPlatformConfigs)) {
            return publicPlatformConfigs;
        }
        return null;
    }

    @Override
    public List<PublicPlatformConfig> getPlatformInfos() {
        List<PublicPlatformConfig> configList = new ArrayList<>();
        PublicPlatformConfig publicPlatformConfig = new PublicPlatformConfig();
        publicPlatformConfig.setIsDeleted(Constants.DEL_FLAG);
        publicPlatformConfig.setType(Constants.ONE);
        EntityWrapper<PublicPlatformConfig> condition = QueryUtil.condition(publicPlatformConfig);
        condition.groupBy("app_id");
        // 微信公众号配置列表
        configList = this.selectList(condition);

        PublicPlatformConfig config = new PublicPlatformConfig();
        config.setIsDeleted(Constants.DEL_FLAG);
        config.setType(Constants.ZERO);
        condition = QueryUtil.condition(publicPlatformConfig);
        condition.groupBy("agent_id");

        List<PublicPlatformConfig> companyConfigList = this.selectList(condition);
        if (CollectionUtils.isNotEmpty(companyConfigList)) {
            configList.addAll(companyConfigList);
        }
        return configList;
    }

    @Override
    public PublicPlatformConfig insertPublicPlatformConfigInfo(PublicPlatformConfig publicPlatformConfig) {

        publicPlatformConfig.setId(idGenerator.generate());
        publicPlatformConfig.setCreateTime(new Date());
        if (StringUtils.isNotBlank(publicPlatformConfig.getAgentId())) {
            publicPlatformConfig.setType(Constants.ZERO);
        } else {
            publicPlatformConfig.setType(Constants.ONE);
        }
        RequestContext context = ContextHolder.get();
        log.info("上下文" + context);
        this.insert(publicPlatformConfig);

        return publicPlatformConfig;

    }

    @Override
    public Page<PublicPlatformConfig> findPublicConfigList(Integer pageNo, Integer pageSize, String name) {
        Page<PublicPlatformConfig> page = new Page<PublicPlatformConfig>(pageNo, pageSize);
        PublicPlatformConfig publicPlatformConfig = new PublicPlatformConfig();
        EntityWrapper condition = QueryUtil.condition(publicPlatformConfig);
        condition.orderBy("create_time", false);
        if (StringUtils.isNotBlank(name)) {
            condition.like("name", name.trim());
        }
        condition.orderBy("create_time",true);
        return this.selectPage(page, condition);
    }

    @Override
    public PublicPlatformConfig getPublicPlatformConfig(PublicPlatformConfig publicPlatformConfig) {

        return this.selectOne(QueryUtil.condition(publicPlatformConfig));
    }
}
