package com.yizhi.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.domain.UserInfo;
import com.yizhi.application.mapper.TrAccountUserinfoMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.ITrAccountUserinfoService;
import com.yizhi.application.service.IUserInfoService;
import com.yizhi.util.application.constant.ReturnCode;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.wechat.application.vo.wechat.TrAccountUserinfoVO;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lilingye
 * @since 2018-04-29
 */
@Log4j2
@Service
public class TrAccountUserinfoServiceImpl extends ServiceImpl<TrAccountUserinfoMapper, TrAccountUserinfo> implements ITrAccountUserinfoService {


    @Autowired
    IdGenerator idGenerator;
    @Autowired
    IUserInfoService userInfoService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public TrAccountUserinfoVO saveAccount(TrAccountUserinfoVO trAccountUserinfoVO) {
        Long companyId = trAccountUserinfoVO.getCompanyId();
        log.info("公司的id为：" + companyId);

        // 查询是否存在 accountId的绑定关系
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setAccountId(trAccountUserinfoVO.getAccountId());
        if (trAccountUserinfoVO.getType() == 5) {
            // 小程序
            trAccountUserinfo.setUnionId(trAccountUserinfoVO.getWechatuserid());
        } else {
            trAccountUserinfo.setWechatuserid(trAccountUserinfoVO.getWechatuserid());
        }
        trAccountUserinfo.setIsDeleted(0);

        TrAccountUserinfo trUserInfoWechat = this.selectOne(QueryUtil.condition(trAccountUserinfo));
        // 检查用户是否已经存在
        if (trUserInfoWechat != null) {
            //如果存在直接返回信息
            BeanUtils.copyProperties(trUserInfoWechat, trAccountUserinfo);

        } else {
            // 如果不存在 查看微信号是和未木云账号绑定或者 账号被其他的微信号绑定

            if (trAccountUserinfoVO.getType() == 5) {
                trAccountUserinfo.setUnionId(null);
            } else {
                trAccountUserinfo.setWechatuserid(null);
            }
            trAccountUserinfo.setCompanyId(companyId);
            TrAccountUserinfo trUserInfo = this.selectOne(QueryUtil.condition(trAccountUserinfo));
            log.info("当前的参数1："+JSON.toJSONString(trAccountUserinfo));
            if (trUserInfo != null) {
                log.info("账号：" + trAccountUserinfoVO.getAccountId() + ",已经绑定过微信账号");
                trAccountUserinfoVO.setCode(ReturnCode.ACCOUNT_BIND.getCode());
                return trAccountUserinfoVO;
            }

            trAccountUserinfo.setAccountId(null);
            trAccountUserinfo.setCompanyId(companyId);
            if (trAccountUserinfoVO.getType() == Constants.FIVE) {
                trAccountUserinfo.setUnionId(trAccountUserinfoVO.getWechatuserid());
            } else {
                trAccountUserinfo.setWechatuserid(trAccountUserinfoVO.getWechatuserid());
            }


            log.info("当前的参数2："+JSON.toJSONString(trAccountUserinfo));
            TrAccountUserinfo wechatUser = this.selectOne(QueryUtil.condition(trAccountUserinfo));

            if (wechatUser != null) {
                log.info("微信账号：" + trAccountUserinfoVO.getWechatuserid() + ",");
                trAccountUserinfoVO.setCode(ReturnCode.OPENID_BIND.getCode());
                return trAccountUserinfoVO;
            }

            if (null == trUserInfo && wechatUser == null) {
                log.info("登录未木云账号未绑定微信账号。");
                Long id = idGenerator.generate();
                trAccountUserinfo.setWechatuserid(trAccountUserinfoVO.getWechatuserid());
                trAccountUserinfo.setId(id);
                trAccountUserinfo.setAccountId(trAccountUserinfoVO.getAccountId());
                trAccountUserinfo.setCompanyId(trAccountUserinfoVO.getCompanyId());
                trAccountUserinfo.setSiteId(trAccountUserinfoVO.getSiteId());
                trAccountUserinfo.setCreateTime(new Date());
                trAccountUserinfo.setType(trAccountUserinfoVO.getType());

                // 获取用户的头像
                UserInfo wechatUserInfo = new UserInfo();
                wechatUserInfo.setWechatuserid(trAccountUserinfoVO.getWechatuserid());
//                String avatar = null;
//                try {
//                    avatar = userInfoService.selectOne(QueryUtil.condition(wechatUserInfo)).getHeadimgurl();
//                    log.info("用户头像:" + avatar);
//                } catch (Exception e) {
//                    e.printStackTrace();
//                    log.info("返回错误{}", e);
//                }

                // 生成wechatuserid和系统用户的关联关系
                Date date = new Date();
                trAccountUserinfo.setCreateTime(date);
                trAccountUserinfo.setUpdateTime(date);
                this.insert(trAccountUserinfo);

                BeanUtils.copyProperties(trAccountUserinfo, trAccountUserinfoVO);
//                trAccountUserinfoVO.setHeadImgUrl(avatar);
                trAccountUserinfoVO.setCode(ReturnCode.SUCCESS.getCode());
                log.info("返回实体：" + trAccountUserinfoVO);
            }
        }
        return trAccountUserinfoVO;

    }

    /**
     * 查询用户是否在未木云用户系统中
     *
     * @param openId 用户唯一标识
     * @return
     */
    @Override
    public TrAccountUserinfo queryAccount(String openId) {

        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setIsDeleted(0);
        trAccountUserinfo.setWechatuserid(openId);

        return this.selectOne(QueryUtil.condition(trAccountUserinfo));
    }

    /**
     * 查找用户是否存在绑定微信关系中
     *
     * @param openId
     * @param companyId
     * @return
     */
    @Override
    public TrAccountUserinfo findAccount(String openId, Long companyId) {
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setIsDeleted(0);
        trAccountUserinfo.setWechatuserid(openId.trim());
        trAccountUserinfo.setCompanyId(companyId);
        List<TrAccountUserinfo> trAccountUserInfoList = this.selectList(QueryUtil.condition(trAccountUserinfo));
        if (CollectionUtils.isEmpty(trAccountUserInfoList)) {
            return null;
        }
        return trAccountUserInfoList.get(0);
    }

    @Override
    public TrAccountUserinfo findBindWechat(TrAccountUserinfo trAccountUserinfo) {
        return this.selectOne(QueryUtil.condition(trAccountUserinfo));
    }

    /**
     * 修改关联关系
     *
     * @param trAccountUserinfo
     * @return
     */
    @Override
    public boolean updateTrAccountUserInfo(TrAccountUserinfo trAccountUserinfo) {

        trAccountUserinfo = this.selectOne(QueryUtil.condition(trAccountUserinfo));
        trAccountUserinfo.setIsDeleted(2);
        return this.updateById(trAccountUserinfo);

    }

    @Override
    public boolean unbindAccountByWeChatUserId(String weChatUserId) {

        if (null == weChatUserId) {
            return false;
        }
        TrAccountUserinfo trAccountUserinfo = new TrAccountUserinfo();
        trAccountUserinfo.setWechatuserid(weChatUserId);
        trAccountUserinfo.setIsDeleted(Constants.DEL_FLAG);
        List<TrAccountUserinfo> accountUserInfoList = this.selectList(QueryUtil.condition(trAccountUserinfo));
        if (CollectionUtils.isNotEmpty(accountUserInfoList)) {
            accountUserInfoList.parallelStream().map(accountUser -> {
                accountUser.setIsDeleted(Constants.YES_FLAG);
                return accountUser;
            }).collect(Collectors.toList());
            boolean b = this.updateBatchById(accountUserInfoList);
        }
        return true;
    }

    @Override
    public boolean updateUserInfo(Long accountId, String weChatUserId) {
        boolean updateResult = false ;

        TrAccountUserinfo accountUserInfo = new TrAccountUserinfo();
        accountUserInfo.setAccountId(accountId);
        accountUserInfo.setIsDeleted(0);
        EntityWrapper<TrAccountUserinfo> condition = QueryUtil.condition(accountUserInfo);
        condition.ne("wechatuserid",weChatUserId);

        List<TrAccountUserinfo> trAccountUserInfoLists = this.selectList(condition);
        if (CollectionUtils.isNotEmpty(trAccountUserInfoLists)) {
            trAccountUserInfoLists = trAccountUserInfoLists.stream().map(trAccountUserInfo -> {
                trAccountUserInfo.setIsDeleted(1);
                return trAccountUserInfo;
            }).collect(Collectors.toList());
            updateResult = this.updateBatchById(trAccountUserInfoLists);
        }


        return updateResult;
    }

    @Override
    public boolean autoUnBindWeChatAccount(TrAccountUserinfo trAccountUserInfo) {
        log.info("当前绑定微信的关系信息："+JSON.toJSON(trAccountUserInfo));

        // 用户accountId
        Long accountId = trAccountUserInfo.getAccountId();
        // 用户wechatId
        String weChatUserId = trAccountUserInfo.getWechatuserid();
        // id
        Long id = trAccountUserInfo.getId();

        TrAccountUserinfo accountUserInfo = new TrAccountUserinfo();
        accountUserInfo.setAccountId(accountId);
        accountUserInfo.setIsDeleted(0);
        EntityWrapper<TrAccountUserinfo> ew = QueryUtil.condition(accountUserInfo);
        ew.ne("type",3);
        ew.ne("id",id);
        List<TrAccountUserinfo> trAccountUserInfoList = this.selectList(ew);

        log.info("返回重复的：{}"+JSON.toJSON(trAccountUserInfoList));

        boolean flag = false;
        if (CollectionUtils.isNotEmpty(trAccountUserInfoList)) {
            List<TrAccountUserinfo> userInfos = trAccountUserInfoList.stream().map(tr -> {
                tr.setIsDeleted(1);
                tr.setUpdateById(1000L);
                tr.setUpdateTime(new Date());
                return tr;
            }).collect(Collectors.toList());
            log.info("返回结果："+userInfos);
            flag = this.updateBatchById(userInfos);
        }

        log.info("返回结果:"+flag);
        return flag;
    }

}
